from . import colors

qss: str = f'''
#ImageButton {{
    background-color: yellow;
    min-width: 120px;
    min-height: 120px;
    max-width: 120px;
    max-height: 120px;
    border-radius: 59px;
    border: none;
}}

#ImageButton QPushButton {{
    background-color: red;
    
}}
'''
